/*
Tool Name: 'facerFX Tools Installer'

Tool by: Mathieson Facer
E-mail: info@mathiesonfacer.com
Website: www.mathiesonfacer.com
Start Date: May 2, 2009

History:
	February 20, 2010		v2.0
		- Cleaned up code.
Description:
	- Installs the tools package.  Adds the plugins path to the plugin.ini file and evaluates all plugins and macros.
To Do:
	- Have to change how the abbreviations file gets installed.  It seems that instead of replacing the file, it needs to be written to.
		- Get the text from my abbreviations file and overwrite what is in the current abbreviations file and save.
Bugs:
	- List of bugs goes here.
Testing:

Test code can be placed in this area.

*/

struct MJF_installer
(
	fn fileInDir dir pattern:"*facerfx*" =
	(
		/*
		<DOC> Performs a fileIn operation on all files using the supplied directory path.
		Arguments:
			<string> dir:	The wildcard directory path to use for the fileIn.
		Return:
			Does not return anything.
		*/
		
		local scriptFiles = getFiles (dir + "\\*.ms")
		local macroFiles = getFiles (dir + "\\*.mcr")
		
		for x in (scriptFiles + macroFiles) where (matchPattern x pattern:pattern) do
		(
			fileIn x
			format "Evaluated %\n" x
		)
	),
	
	fn copyDir sourcePath destPath =
	(
		/*
		<DOC> Copies files from the sourcePath to the destPath.
		Arguments:
			<string> sourcePath:	The directory to copy scripts from.
			<string> destPath:	The directory to copy scripts to.
		Return:
			Does not return anything.
		*/
		
		if not(doesFileExist destPath) then
		(
			makeDir destPath
		)
		
		local scriptFiles = getFiles (sourcePath + "\\*.ms")
		local macroFiles = getFiles (sourcePath + "\\*.mcr")
		
		for x in (scriptFiles + macroFiles) do
		(
			copyFile x (destPath + "\\" + (filenameFromPath x))
			format "Copied % to %\n" (filenameFromPath x) destPath
		)
		
		local subDirs = getDirectories (sourcePath +"\\*")
		for x in subDirs do
		(
			local pathPieces = filterString x "\\"
			local newDest = destPath + "\\" + pathPieces[pathPieces.count]
			copyDir x newDest
		)
	),
	
	fn deleteFiles dir pattern:"*facerFX*" =
	(
		/*
		<DOC> Recursively deletes any files with "facerFX" found in the file path string.
		Arguments:
			<string> dir:	The directory to process.
		Return:
			Does not return anything.
		*/
		
		local files = getFiles (dir + "\\*.*")
		local subDirs = getDirectories (dir + "\\*")
		
		for x in files where (matchPattern x pattern:pattern) do
		(
			if deleteFile x then
			(
				format "Removed file:\t%\n" x
			)
			else
			(
				format "Could not remove file:\t%\n" x
			)
		)
		
		for x in subDirs do
		(
			deleteFiles x pattern:pattern
		)
	),
	
	fn startInstall =
	(
		/*
		<DOC> Sets the plugin path for the tools and performs a fileIn for all plugins and macros.
		Arguments:
			Does not require any arguments.
		Return:
			Does not return anything.
		*/
		
		local tempDir = getDir #temp
		
		local scriptsDir = getDir #userScripts
		copyDir (tempDir + "\\scripts\\") scriptsDir
		local pluginsDir = scriptsDir + "\\facerFX_Tools\\plugins\\"
		fileInDir pluginsDir
		
		local macrosDir = getDir #userMacros
		copyDir (tempDir + "\\ui\\usermacros\\") macrosDir
		fileInDir (macrosDir + "\\facerFX_Tools")
		deleteFiles macrosDir pattern:"*facerfx*-*"
		
		local pluginIni = (getDir #maxData) + "\\plugin.ini"
		if doesFileExist pluginIni then
		(
			setINISetting pluginIni "Directories" "facerFX_Tools" pluginsDir
		)
		
		messagebox "Installation successful." title:"facerFX Tools"
	)
)
MJF_installer = MJF_installer()
MJF_installer.startInstall()